<?php
    if(!function_exists('LPC_autoloader'))
    {
        function LPC_autoloader($class)
        {
            if (strpos($class, 'LPC_') !== false):
                if (!class_exists('LPLib/' . $class, false)):
                    #doesnt exist so include it
                    include_once __DIR__.DS.'LPLib' . DS . $class . '.php';
                endif;
            endif;

        }
    }
    spl_autoload_register('LPC_autoloader');

    class Litepay extends PaymentGatewayModule
    {
        function __construct()
        {
            $this->name             = __CLASS__;
            parent::__construct();
            $this->define_function('ipn');
        }

        public function config_fields()
        {
            return [
                'litepay_secret_key'          => [
                    'name'              => "SecretKey",
                    'description'       => 'Type in the <b>SecretKey</b> of your merchant account.',
                    'type'              => "Text",
                    'options'           => '',
                    'value'             => $this->config["settings"]["litepay_secret_key"] ?? '',
                ],
                'litepay_merchant_id'          => [
                    'name'              => "MerchantId",
                    'description'       => 'Type in the <b>Merchant ID</b> of your merchant account.',
                    'type'              => "Text",
                    'options'           => '',
                    'value'             => $this->config["settings"]["litepay_merchant_id"] ?? '',
                ]
            ];
        }

        public function area($params=[])
        {

            // Invoice Parameters
            $invoiceId = $this->checkout_id;
            $description = 'Invoice Payment';
            $amount = $params['amount'];
            $currencyCode = $params['currency'];
            // Client Parameters
            $firstname  = $this->clientInfo->name;
            $lastname   = $this->clientInfo->surname;
            $email      = $this->clientInfo->email;
            $address1   = $this->clientInfo->address->address;
            $address2   = '';
            $city       = $this->clientInfo->address->city;
            $state      = $this->clientInfo->address->counti;
            $postcode   = $this->clientInfo->address->zipcode;
            $country    = $this->clientInfo->address->country_code;
            $phone      = $this->clientInfo->phone;

            // System Parameters

            $returnUrl = $this->links['return'];
            $langPayNow = $this->l_payNow;

            $url                      = '';


            $postfields = array();
            $postfields['username'] = '';
            $postfields['invoice_id'] = $invoiceId;
            $postfields['description'] = $description;
            $postfields['amount'] = $amount;
            $postfields['currency'] = $currencyCode;
            $postfields['first_name'] = $firstname;
            $postfields['last_name'] = $lastname;
            $postfields['email'] = $email;
            $postfields['address1'] = $address1;
            $postfields['address2'] = $address2;
            $postfields['city'] = $city;
            $postfields['state'] = $state;
            $postfields['postcode'] = $postcode;
            $postfields['country'] = $country;
            $postfields['phone'] = $phone;
            $postfields['return_url'] = $returnUrl;

            $htmlOutput = '<form method="post" action="' . $url . '">';
            foreach ($postfields as $k => $v) {
                $htmlOutput .= '<input type="hidden" name="' . $k . '" value="' . urlencode($v) . '" />';
            }

            $wh = new LPC_Wh();
            $litepay_checkout_token = $wh->LPC_getLitepayToken($this->config['settings']['litepay_secret_key'], $this);
            $litepay_merchant_id = $wh->LPC_getLitepayMid($this->config['settings']['litepay_merchant_id'], $this);

            $config = new LPC_Configuration($litepay_checkout_token, $litepay_merchant_id, 'production');
            $params = new stdClass();

            $callback_url               = $this->links["callback"].'?invoice_id='.$invoiceId.'&secret='.$this->config['settings']['litepay_secret_key'];
            $params->extension_version  = 'Litepay_Checkout_WISECP'. '_1.0.0';
            $params->price = $amount;
            $params->currency = $currencyCode;
            $params->orderId = trim($invoiceId);
            $params->redirectURL = $this->links["return"];
            $params->extendedNotifications = true;
            $params->callbackurl = $callback_url;

            if (!empty($email)):
                $buyerInfo = new stdClass();
                $buyerInfo->name = $firstname . ' ' . $lastname;
                $buyerInfo->email = $email;
                $params->buyer = $buyerInfo;
            endif;

            $item = new LPC_Item($config, $params);
            $invoice = new LPC_Invoice($item);
            //this creates the invoice with all of the config params from the item
            
            $invoice->LPC_createInvoice();
            $invoiceData = $invoice->LPC_getInvoiceData();
            $invoiceID = $invoice->LPC_getInvoiceID();
            $invoiceURL = $invoice->LPC_getInvoiceURL();
            
            $this->save_custom_data([
                'transaction_id'        => $invoiceID,
                'transaction_status'    => 'new',
                'transaction_time'      => gmdate("Y-m-d H:i:s"),
                ]);

            $htmlOutput .= '<div align="center"> <p> Click the button below to be redirected to our crypto currencies merchant </p> <p> Once the payment has been made & confirmed, your account will reflect the chosen services </p>
            <button name = "litepay-payment" class = "lbtn green"  onclick="redirectToUrl(\''. $invoiceURL .'\');return false;">' . $langPayNow . '</button></div>';
            ?>
            
            <?php

            $htmlOutput .= '
            <script>
                function redirectToUrl(url) {
                    window.location=url;
              }
            </script>
            </form>';
            return $htmlOutput;

        }

        public function callback()
        {

            $invoiceId           = Filter::GET("invoice_id");
            $secretID            = Filter::GET("secretid");
            $destination_address = Filter::GET("destination_address");
            $invoice_address     = Filter::GET("input_address");
            $invoice_conf        = Filter::GET("confirmations");
            $invoice_value       = Filter::GET("value");
            $invoice_trans_hash  = Filter::GET("transaction_hash");
            $invoice_coin        = Filter::GET("coin");

            if(!$invoiceId){
                $this->error = 'ERROR: invoiceId id is empty!';
                return false;
            }

            if(!$secretID){
                $this->error = 'ERROR: secret id is empty!';
                return false;
            }

            $callback = new LPC_Callback($secretID, $destination_address, $invoice_address, $invoiceId, $invoice_conf, $invoice_value, $invoice_trans_hash, $invoice_coin);

            if ($this->config['settings']['litepay_secret_key'] === $secretID) {
                $checkout = $this->get_checkout($callback->LPC_getCinvoiceid());
                switch ($checkout['status']) {
                    case "paid":
                        return [
                            'callback_message'        => '*ok*',
                        ];
                        break;
                    case "unpaid":
                        $note = "Received payment of ". $callback->LPC_getCinvoicecryptovalue() ."/sats ".$callback->LPC_getCCoin()." trough address ". $callback->LPC_getCinvoiceaddress() ." to ". $callback->LPC_getCDestinationAddress() . " TxID: ". $callback->LPC_getCinvoicetransactionhash() ." , current confirmations: ". $callback->LPC_getCinvoiceconfirmations() ." for transaction_id: ".$callback->LPC_getCinvoiceid();
                        $this->set_checkout($checkout);
                        $custom_data        = $this->get_custom_data();
                        $custom_data['transaction_id']     = $callback->LPC_getCinvoiceid();
                        $custom_data['transaction_status'] = 'paid';
                        $custom_data['transaction_update_time']   = gmdate("Y-m-d H:i:s");
                        $custom_data['transaction_note']   = $note;
                        $this->save_custom_data($custom_data);

                        return [
                            'status'            => 'pending',
                            'message'        => [
                                'Transaction ID' => $callback->LPC_getCinvoiceid(),
                                'Coin ' => $callback->LPC_getCCoin(),
                                'Value' => $callback->LPC_getCinvoicecryptovalue(). ' sats',
                                'Input Address' => $callback->LPC_getCinvoiceaddress(),
                                'TX id' => $callback->LPC_getCinvoicetransactionhash(),
                               ],
                            'callback_message'        => '*ok*',
                        ];
                        break;
                    default:
                        exit('Wrong place for you to be.'); 
                }
            } else {
                error_log('Invalid secret key request'.$secretID);
                echo "Invalid request!";
                exit();
            }
        }

    }